#include <libpv_config.h>

#include <errno.h>
#include <getopt.h>
#include <libgen.h>
#include <string.h>

#include <pv_types.h>
#include <libpv.h>
#include <jv_server.h>


int pv_engine_attr_init(pv_engine_attr_t	*attr)
{
	if (!attr)
		return -EINVAL;

	memset(attr, 0, sizeof(struct pv_engine_attr));

	attr->store_type = PV_STORE_STANDARD;

	return 0;
}


static void show_help(char *name)
{
	fprintf(stderr,
		"\nusage:   "
		"%s [-v] -d <file> -s <file> [-j <port>]\n\n"
		"  -v, --verbose             Verbose mode of operation\n"
		"  -d, --description <file>  Description XML File: specifies layout of the \n"
		"                            process variable store file\n"
		"  -s, --store <file>        Store file: this file holds the process variables\n"
		"  -8, --8bit                use 8 bit backing store\n"
		"  -j, --jvisu [<port>]      Start JVisu Socket Server, default %d\n"
		"\n",
		name, JVISU_PORT_DEFAULT);
}


int pv_engine_attr_init_from_cmdline(pv_engine_attr_t	*attr,
				     int		argc,
				     char		*argv[])
{
	struct option	long_options[] = {
		{ "help",       no_argument,       0, 'h' },
		{ "verbose",    no_argument,       0, 'v' },
		{ "desc", 	required_argument, 0, 'd' }, 
		{ "store",      required_argument, 0, 's' },
		{ "8bit",       no_argument,       0, '8' },
		{ 0,            0,                 0,  0  },
	};
	char		*optdesc = NULL;
	char		*optstore = NULL;
	char		*optjvport = NULL;
	int		longindex;
	int		opt;

	if (!attr)
		return -EINVAL;

	memset(attr, 0, sizeof(struct pv_engine_attr));

	/* reset getopt */
	optind = 0;

	/* parse command line switches */
	while ((opt = getopt_long(argc, argv, "hvd:s:8j::", long_options, &longindex)) != -1) {
		switch (opt) {
		case 'h':
			show_help(basename(argv[0]));
			return -1;
		case 'v':
			_libpv_debuglevel++;	/* FIXME: move this into pv_engine struct (mkl) */
			break;
		case 'd':
			optdesc = optarg;
			break;
		case 's':
			optstore = optarg;
			break;
		case 'j':
			optjvport = optarg;
			break;
		case '8':
			attr->store_type = PV_STORE_8BIT;
			break;
                default:
                        show_help(argv[0]);
			return -EINVAL;
                }
	}

	if (!optdesc || !optstore) {
		show_help(argv[0]);
		return -EINVAL;
	}

	if (optjvport)
		attr->jvisu_port = strtoul(optjvport, NULL, 0);
	else
		attr->jvisu_port = JVISU_PORT_DEFAULT;

	strncpy(attr->store_filename, optstore, PV_PATH_MAX);
	attr->store_filename[PV_PATH_MAX] = 0;

	strncpy(attr->desc_filename, optdesc, PV_PATH_MAX);
	attr->desc_filename[PV_PATH_MAX] = 0;

	return 0;
}
