#include <libpv_config.h>

#include <errno.h>

#include <pv_lock.h>
#include <libpv.h>


int pv_lock_init(struct pv_engine	*engine,
		 const char		*store_filename)
{
	int	err;
	/* FIXME: use absolute paths (sha) */

	engine->pv_sem_key = ftok(store_filename, 42); 
	engine->pv_sem = semget(engine->pv_sem_key, 1,
				S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH
				| IPC_CREAT | IPC_EXCL);

	if (engine->pv_sem < 0 && errno == EEXIST) {		/* exists already */
		engine->pv_sem = semget(engine->pv_sem_key, 1,
					S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH
					| IPC_CREAT);

		if (engine->pv_sem < 0) {
			err = -errno;
			dprintf(LEVEL_CRIT, "(%s) could not initialize semaphore, err %d\n",
				__FUNCTION__, err);
			goto exit_0;
		}

	} else if (engine->pv_sem >= 0) {			/* newly created */
		union semun	sem_val = { .val = 1 }; 

		err = semctl(engine->pv_sem, 0, SETVAL, sem_val);
		if (err < 0) {
			err = -errno;
			dprintf(LEVEL_CRIT, "(%s) could not initialize semaphore, err %d\n",
				__FUNCTION__, err);
			goto exit_0;
		}

	} else {						/* failed */
		err = -errno;
		dprintf(LEVEL_CRIT, "(%s) could not initialize semaphore, err %d\n",
			__FUNCTION__, err);
		goto exit_0;	/* FIXME: free sema (mkl) */
	}
	dprintf(LEVEL_INFO, "(%s) sem_id %d\n", __FUNCTION__, engine->pv_sem);

	return 0;

 exit_0:
	return err;
}


int pv_lock_destroy(struct pv_engine	*engine)
{
	/* FIXME: do we need to remove the SysV semaphore? */
	/* SHA: Yes, but make sure we are the last process using it */

	return 0;
}
