#include <libpv_config.h>

#include <errno.h>
#include <string.h>

#include <libpv.h>
#include <pv_register.h>


char *pv_type_str[] = {
	[PV_TYPE_INT8]	 = "int8_t",
	[PV_TYPE_UINT8]  = "uint8_t",

	[PV_TYPE_INT16]  = "int16_t",
	[PV_TYPE_UINT16] = "uint16_t",

	[PV_TYPE_INT32]  = "int32_t",
	[PV_TYPE_UINT32] = "uint32_t",

	[PV_TYPE_INT64]  = "int64_t",
	[PV_TYPE_UINT64] = "uint64_t",

	[PV_TYPE_STRING] = "string",

};


struct pv
*pv_register_lookup(struct pv_engine	*pv_engine,
		    const char		*id)
{
	struct pv		*pv;
	int			finished = 0; 

	for (goto_head(&pv_engine->pv_list); !finished; goto_next(&pv_engine->pv_list)) {
		pv = (struct pv *)list_get_element_data(pv_engine->pv_list);
			
		if (is_last(pv_engine->pv_list))
			finished=1;

		if (!strcmp(id, pv->id)) {
			return pv;
		}
	}

	dprintf(LEVEL_DEBUG, "(%s) variable with id \"%s\" not found\n",
		__PRETTY_FUNCTION__, id);

	errno = ENXIO;
	return NULL;
}


static struct pv
*pv_register_lookup_check(struct pv_engine	*pv_engine,
			  const char		*id,
			  enum pv_type		type)
{
	struct pv	*pv;

	pv = pv_register_lookup(pv_engine, id);
	if (!pv)
		return NULL;

	if (pv->type != type) {
		dprintf(LEVEL_DEBUG, "(%s) variable \"%s\" requested type %s but is %s\n",
			__PRETTY_FUNCTION__,
			id,
			pv_type_str[type],	
			pv_type_str[pv->type]);
		errno = ENOTSUP;
		return NULL;
	}

	return pv;
}


uint8_t *pv_register_uint8(pv_engine_t	*pv_engine_anon,
			   const char	*id)
{
	struct pv_engine	*pv_engine = (struct pv_engine *)pv_engine_anon;
	struct pv		*pv;

	if (!pv_engine || !id)
		return NULL;

	pv = pv_register_lookup_check(pv_engine, id, PV_TYPE_UINT8);
	if (!pv)
		return NULL;
	
	return pv_to_abs_addr(pv_engine, pv)->u8;
}


uint16_t *pv_register_uint16(pv_engine_t	*pv_engine_anon,
			     const char		*id)
{
	struct pv_engine	*pv_engine = (struct pv_engine *)pv_engine_anon;
	struct pv		*pv;

	if (!pv_engine || !id)
		return NULL;

	pv = pv_register_lookup_check(pv_engine, id, PV_TYPE_UINT16);
	if (!pv)
		return NULL;
	
	return pv_to_abs_addr(pv_engine, pv)->u16;
}


uint32_t *pv_register_uint32(pv_engine_t	*pv_engine_anon,
			     const char		*id)
{
	struct pv_engine	*pv_engine = (struct pv_engine *)pv_engine_anon;
	struct pv		*pv;

	if (!pv_engine || !id)
		return NULL;

	pv = pv_register_lookup_check(pv_engine, id, PV_TYPE_UINT32);
	if (!pv)
		return NULL;
	
	return pv_to_abs_addr(pv_engine, pv)->u32;
}



char *pv_register_string(pv_engine_t	*pv_engine_anon,
			 const char	*id)
{
	struct pv_engine	*pv_engine = (struct pv_engine *)pv_engine_anon;
	struct pv		*pv;

	if (!pv_engine || !id)
		return NULL;

	pv = pv_register_lookup_check(pv_engine, id, PV_TYPE_STRING);
	if (!pv)
		return NULL;
	
	return pv_to_abs_addr(pv_engine, pv)->string;
}



